/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package core;

import Jama.EigenvalueDecomposition;
import Jama.Matrix;

/**
 *
 * @author pchr
 */
public class Frame {
    private Element[] FrameElements;
    private Node[] FrameNodes; 
    private Matrix K;
    private Matrix M;
    private int[][] dofs;
    private final int nElems=9;
    private final int nNodes=8;
    private double[] EigenMasses;
    private double[] ContributionCoefs;
    private double[] Sa;
    private boolean stiffFrame=false;
    
    double[] eig_vals;
    private Matrix eigenMat;
    private double scale=1.00;
    
    private double h1,h2,h3;
    private double m1,m2,m3;
    private double b1,b2,b3;
    private double d1,d2,d3;
    private double L,Elast;
    double I1,I2,I3;
    double A1,A2,A3;
    
    private EigenvalueDecomposition EVofInvM_A ;
    
    
    public Frame(){
        this.FrameElements = new Element[nElems];
        this.FrameNodes = new Node[nNodes];
        this.setDOFS();
    }
    
    public void addElement(int wElem, Element anElement){
        this.FrameElements[wElem-1]=anElement;
    }
    
    public void addNode(int wNode, Node aNode){
        this.FrameNodes[wNode-1]=aNode;
    }
    
    public Element getElement(int wElem){
        return this.FrameElements[wElem-1];
    }
    
    public Element[] getElements(){
        return this.FrameElements;
    }
    
    public Node getNode(int wNode){
        return this.FrameNodes[wNode-1];
    }
    
    public Node[] getNodes(){
        return this.FrameNodes;
    }
    
    
    public void setK(){
        K = new Matrix(3,3);
        K.init();
        double cf;
        if(this.stiffFrame){
            cf=12;
            K.addVal(0, 0, 2.*cf*Elast*I1/(h1*h1*h1)+2.*cf*Elast*I2/(h2*h2*h2));
            K.addVal(1, 0, -2.*cf*Elast*I2/(h2*h2*h2));
            K.addVal(2, 0, 0.0);
            K.addVal(0, 1, -2.*cf*Elast*I2/(h2*h2*h2));
            K.addVal(1, 1, 2.*cf*Elast*I3/(h3*h3*h3)+2.*cf*Elast*I2/(h2*h2*h2));
            K.addVal(2, 1, -2.*cf*Elast*I3/(h3*h3*h3));
            K.addVal(0, 2, 0.0);
            K.addVal(1, 2, -2.*cf*Elast*I3/(h3*h3*h3));
            K.addVal(2, 2, 2.*cf*Elast*I3/(h3*h3*h3));
        }else{
            K.addVal(0, 0,2.*3.*Elast*(3.*h2*h2*h2*h2*I1*I1*I3+12.*h1*h1*h2*I1*I2*(h3*I2+h1*I3)+4.*h2*h2*h2*I1*I2*(h3*I1+3.*h1*I3)+h1*h1*h1*I2*I2*(4*h3*I1+3*h1*I3)+6*h1*h2*h2*I1*I2*(2*h3*I2+3*h1*I3))/(h1*h1*h1*h2*h2*(h3*I2*(4*h2*I1+3*h1*I2)+3*h2*(h2*I1+h1*I2)*I3)));
            K.addVal(1, 0,-2.*3*Elast*I2*(4*h1*h3*h3*I1*I2+6*h1*h2*(h2+2*h3)*I1*I3+3*h1*h1*(h2+h3)*I2*I3+3*h2*I1*(2*h3*h3*I2+h2*(h2+3*h3)*I3))/(h1*h2*h2*h3*(h3*I2*(4*h2*I1+3*h1*I2) +3*h2*(h2*I1+h1*I2)*I3)) );
            K.addVal(2, 0,2.*9*Elast*I2*(h2*(2*h1+h2)*I1+h1*h1*I2)*I3/(h1*h2*h3*(h3*I2*(4*h2*I1+3*h1*I2)+3*h2*(h2*I1+h1*I2)*I3)) );
            K.addVal(0, 1,-2.*3*Elast*I2*(4*h1*h3*h3*I1*I2+6*h1*h2*(h2+2*h3)*I1*I3+3*h1*h1*(h2+h3)*I2*I3+3*h2*I1*(2*h3*h3*I2+h2*(h2+3*h3)*I3))/(h1*h2*h2*h3*(h3*I2*(4*h2*I1+3*h1*I2) +3*h2*(h2*I1+h1*I2)*I3))  );
            K.addVal(1, 1,2.*3*Elast*I2*(4*h3*h3*h3*I1*I2+6*h2*h3*(2*h2*I1+h1*I2)*I3+3*h3*h3*(4*h2*I1+h1*I2)*I3+h2*h2*(4*h2*I1+3*h1*I2)*I3)/(h2*h2*h3*h3*(h3*I2*(4*h2*I1+3*h1*I2)+3*h2*(h2*I1+h1*I2)*I3)) );
            K.addVal(2, 1,-2.*3*Elast*I2*(2*h2*(2*h2+3*h3)*I1+3*h1*(h2+h3)*I2)*I3/(h2*h3*h3*(h3*I2*(4*h2*I1+3*h1*I2)+3*h2*(h2*I1+h1*I2)*I3)) );
            K.addVal(0, 2,2.*9*Elast*I2*(h2*(2*h1+h2)*I1+h1*h1*I2)*I3/(h1*h2*h3*(h3*I2*(4*h2*I1+3*h1*I2)+3*h2*(h2*I1+h1*I2)*I3))  );
            K.addVal(1, 2,-2.*3*Elast*I2*(2*h2*(2*h2+3*h3)*I1+3*h1*(h2+h3)*I2)*I3/(h2*h3*h3*(h3*I2*(4*h2*I1+3*h1*I2)+3*h2*(h2*I1+h1*I2)*I3))  );
            K.addVal(2, 2,2.*3*Elast*I2*(4*h2*I1+3*h1*I2)*I3/(h3*h3*(h3*I2*(4*h2*I1+3*h1*I2)+3*h2*(h2*I1+h1*I2)*I3)));
        }
    }
    
    public void setM(){
        M = new Matrix(3,3);
        M.init();
        M.addVal(0, 0, m1);
        M.addVal(1, 1, m2);
        M.addVal(2, 2, m3);
    }
    
    private void setDOFS(){
        dofs= new int[9][6];
        dofs[0][0]=1; dofs[0][1]=2; dofs[0][2]=3; dofs[0][3]=7; dofs[0][4]=8; dofs[0][5]=9;
        dofs[1][0]=4; dofs[1][1]=5; dofs[1][2]=6; dofs[1][3]=10; dofs[1][4]=11; dofs[1][5]=12;
        dofs[2][0]=7; dofs[2][1]=8; dofs[2][2]=9; dofs[2][3]=10; dofs[2][4]=11; dofs[2][5]=12;
        dofs[3][0]=7; dofs[3][1]=8; dofs[3][2]=9; dofs[3][3]=13; dofs[3][4]=14; dofs[3][5]=15;
        dofs[4][0]=10; dofs[4][1]=11; dofs[4][2]=12; dofs[4][3]=16; dofs[4][4]=17; dofs[4][5]=18;
        dofs[5][0]=13; dofs[5][1]=14; dofs[5][2]=15; dofs[5][3]=16; dofs[5][4]=17; dofs[5][5]=18;
        dofs[6][0]=13; dofs[6][1]=14; dofs[6][2]=15; dofs[6][3]=19; dofs[6][4]=20; dofs[6][5]=21;
        dofs[7][0]=16; dofs[7][1]=17; dofs[7][2]=18; dofs[7][3]=22; dofs[7][4]=23; dofs[7][5]=24;
        dofs[8][0]=19; dofs[8][1]=20; dofs[8][2]=21; dofs[8][3]=22; dofs[8][4]=23; dofs[8][5]=24;

    }
    
    public void EigenRun(){
        Matrix eVec = new Matrix(3,1);
        Matrix delta = new Matrix(3,1);
        delta.init();
        delta.set(0, 0, 1.);
        delta.set(1, 0, 1.);
        delta.set(2, 0, 1.);
         K=K.inverse();
         M=K.times(M);  
         this.EVofInvM_A = new EigenvalueDecomposition(M);
         eig_vals = new double[3];
         eigenMat = new Matrix(3,3);
         this.setK();
         this.setM();
         for(int i=0;i<3;i++){
             eig_vals[i]=EVofInvM_A.getRealEigenvalues()[i];
             eigenMat.set(0, i, EVofInvM_A.getV().get(0, i));
             eigenMat.set(1, i, EVofInvM_A.getV().get(1, i));
             eigenMat.set(2, i, EVofInvM_A.getV().get(2, i));
         }
         //eig_vals=EVofInvM_A.getRealEigenvalues();
         //eigenMat=EVofInvM_A.getV();
         // SORT
         int e1,e2,e3;
         double max=eig_vals[0];
         e1=0;e2=0;e3=0;
         for(int i=0;i<3;i++){
             if(eig_vals[i]>max){max=eig_vals[i]; e1=i;}
         }
         
         if(e1==0){
             if(eig_vals[1]>eig_vals[2]){e2=1;e3=2;}
             if(eig_vals[2]>eig_vals[1]){e2=2;e3=1;}
         }
         if(e1==1){
             if(eig_vals[0]>eig_vals[2]){e2=0;e3=2;}
             if(eig_vals[2]>eig_vals[0]){e2=2;e3=0;}
         }
         if(e1==2){
             if(eig_vals[0]>eig_vals[1]){e2=0;e3=1;}
             if(eig_vals[1]>eig_vals[0]){e2=1;e3=0;}
         }
         eig_vals[0]=EVofInvM_A.getRealEigenvalues()[e1];
         eig_vals[1]=EVofInvM_A.getRealEigenvalues()[e2];
         eig_vals[2]=EVofInvM_A.getRealEigenvalues()[e3];
         
         eigenMat.set(0, 0, EVofInvM_A.getV().get(0, e1)/EVofInvM_A.getV().get(2, e1));
         eigenMat.set(1, 0, EVofInvM_A.getV().get(1, e1)/EVofInvM_A.getV().get(2, e1));
         eigenMat.set(2, 0, EVofInvM_A.getV().get(2, e1)/EVofInvM_A.getV().get(2, e1));
         
         eigenMat.set(0, 1, EVofInvM_A.getV().get(0, e2)/EVofInvM_A.getV().get(2, e2));
         eigenMat.set(1, 1, EVofInvM_A.getV().get(1, e2)/EVofInvM_A.getV().get(2, e2));
         eigenMat.set(2, 1, EVofInvM_A.getV().get(2, e2)/EVofInvM_A.getV().get(2, e2));
         
         eigenMat.set(0, 2, EVofInvM_A.getV().get(0, e3)/EVofInvM_A.getV().get(2, e3));
         eigenMat.set(1, 2, EVofInvM_A.getV().get(1, e3)/EVofInvM_A.getV().get(2, e3));
         eigenMat.set(2, 2, EVofInvM_A.getV().get(2, e3)/EVofInvM_A.getV().get(2, e3));
         
         this.EigenMasses = new double[3];
         this.ContributionCoefs = new double[3];
         
         for(int i_eig=0; i_eig<3; i_eig++){
             eVec = eigenMat.getMatrix(0, 2, i_eig, i_eig);
             EigenMasses[i_eig]=eVec.transpose().times(M).times(eVec).get(0, 0);
             ContributionCoefs[i_eig]=eVec.transpose().times(M).times(delta).get(0, 0)/EigenMasses[i_eig];
         }
     }
    
    
    private void equal(Matrix a, Matrix b){
        for(int i=0;i<a.getRowDimension();i++){b.set(i, 0, a.get(i, 0));}
    }
    
    private void equal(Matrix a, Matrix b, int row){
        for(int i=0;i<a.getRowDimension();i++){b.set(i, row, a.get(i, 0));}
    }
    
    
    public double[] getEigenValues(){
        //eigenMat.print(12, 6);
        return this.eig_vals;
    }
    
    public double getL(){
        return L;
    }
    
    public double geth1(){
        return h1;
    }
    
    public double geth2(){
        return h2;
    }
    
    public double geth3(){
        return h3;
    }

    
    public Matrix getDefElement(int elemId, double x0, int sit){
        // x0 e [-1,1]
        double u1,v1,f1,u2,v2,f2;
        u1=0.;v1=0.;f1=0.;u2=0.;v2=0.;f2=0.;
        double[] vec = new double[24];
        double[] send_vec = new double[3];
        if(sit<=3){
            for(int i=0;i<3;i++){
                send_vec[i]=eigenMat.get(i, sit-1);
            }
        }
        double max=0.;
        for(int i=0;i<3;i++){
            if(Math.abs(send_vec[i])>max){max=Math.abs(send_vec[i]);}
        }
        
        for(int i=0;i<3;i++){
            send_vec[i]=scale*send_vec[i]/max;
        }
        for(int i=0;i<vec.length;i++){vec[i]=0.;}
        vec[6]=send_vec[0]; vec[9]=send_vec[0];
        vec[12]=send_vec[1]; vec[15]=send_vec[1];
        vec[18]=send_vec[2]; vec[21]=send_vec[2];
        
        u1=vec[dofs[elemId-1][0]-1];
        v1=vec[dofs[elemId-1][1]-1];
        f1=vec[dofs[elemId-1][2]-1];
        u2=vec[dofs[elemId-1][3]-1];
        v2=vec[dofs[elemId-1][4]-1];
        f2=vec[dofs[elemId-1][5]-1];
        /*if(elemId==1 || elemId==2){
            f2=-(u2-u1)/this.getL();
        }else{
            f1=-(u2-u1)/this.getL();
            f2=f1;
        }*/
        return this.FrameElements[elemId-1].getDef(u1, v1, f1, u2, v2, f2, x0);
    }
    
    public double getEigPeriod(int which){
        double ep=0.;
        if(which<=3 && which>0 && eig_vals!=null){
            ep=1./eig_vals[which-1];
            ep=Math.sqrt(ep);
            ep=2.*Math.PI/ep;
        }
        return ep;
    }
    
    public double getEigMass(int which){
        double ep=0.;
        if(which<=3 && which>0){
            ep=this.EigenMasses[which-1];
        }
        return ep;
    }
    
    public double getEigContribution(int which){
        double ep=0.;
        if(which<=3 && which>0){
            ep=this.ContributionCoefs[which-1];
        }
        return ep;
    }
    
    public double getScale(){
        return this.scale;
    }
    
    public void setScale(double scale){
        this.scale=scale;
    }
    
    
    public void setGeometry(double[] data ){
        h1=data[0]; h2=data[1]; h3=data[2];
        m1=data[3]; m2=data[4]; m3=data[5];
        b1=data[6]; b2=data[7]; b3=data[8];
        d1=data[9]; d2=data[10]; d3=data[11];
        L=data[12]; Elast=data[13];
        A1=b1*d1; A2=b2*d2; A3=b3*d3;
        I1=b1*d1*d1*d1/12.; I2=b2*d2*d2*d2/12.; I3=b3*d3*d3*d3/12.;
    }
    
    public void setStiff(boolean st){
        this.stiffFrame=st;
    }
    
    public void setSa(double[] vals){
        Sa = new double[3];
        this.Sa[0]=vals[0];
        this.Sa[1]=vals[1];
        this.Sa[2]=vals[2];
    }
    
    public double getSa(int which){
        double val;
        val=Sa[which-1];
        return val;
    }
    
    public double getSd(int which){
        double val;
        val=Sa[which-1]/((2.*Math.PI/getEigPeriod(which))*(2.*Math.PI/getEigPeriod(which)));
        return val;
    }
    
    public double getSv(int which){
        double val;
        val=Sa[which-1]/((2.*Math.PI/getEigPeriod(which)));
        return val;
    }
    
    public double[] getEigenVector(int which){
        double[] vec = new double[3];
        vec[0]=this.eigenMat.get(0, which-1);
        vec[1]=this.eigenMat.get(1, which-1);
        vec[2]=this.eigenMat.get(2, which-1);
        return vec;
    }
    
    public Matrix getK(){
        return this.K;
    }
    
    public Matrix getM(){
        return this.M;
    }
}
